import os
import shutil
import sys
from time import sleep
import serial.tools.list_ports
import tkinter as tk
from glob import glob
from platform import system
from threading import Thread
from tkinter import filedialog, Button, Text

from .Config import Config
from .Device import Device

baud_rate = 9600


def define_os_specific_params():
    global serial_port_array
    global initialdir
    operating_system = system()
    match operating_system:
        case "Linux":
            serial_port_array = glob("/dev/ttyACM*")
            initialdir = "~/Desktop"
        case "Windows":
            def get_ports():
                ports = serial.tools.list_ports.comports()
                return [port.device for port in ports]
            serial_port_array = get_ports()
            initialdir = "~\\Desktop"


def import_config(console_output):
    filename = filedialog.askopenfilename(initialdir=initialdir,
                                          filetypes=[("Text files", "*.txt"),
                                                     ("Config files", "*.cfg")])
    if filename:
        destination_dir = os.path.join(os.path.dirname(__file__), "config")
        os.makedirs(destination_dir, exist_ok=True)
        destination_file = os.path.join(destination_dir, "config.cfg")
        try:
            shutil.copy(filename, destination_file)
            console_output.insert(tk.END, "Config file imported successfully.\n")
        except Exception as e:
            console_output.insert(tk.END, "Error:" + str(e) + "\n")
    else:
        console_output.insert(tk.END, "No file selected.\n")


def serial_parallel_process(target):
    threads = []
    for serial_port in serial_port_array:
        thread = Thread(target=target, args=(serial_port, baud_rate))
        threads.append(thread)
        thread.start()
    for thread in threads:
        thread.join()


def discrepancy_parallel_process(target, console_output):
    threads = []
    for serial_port in serial_port_array:
        thread = Thread(target=target, args=(serial_port, baud_rate, console_output))
        threads.append(thread)
        thread.start()
    return threads


def config_process(console_output) -> None:
    # Hacky way of detecting new badges after the program has already started
    define_os_specific_params()

    serial_parallel_process(target=Device.start_dev)
    sleep(5)

    serial_parallel_process(target=Device.set_config_on_device)

    discrepancy_parallel_process(target=Config.check_config_discrepancy, console_output=console_output)
    sleep(5)

    serial_parallel_process(target=Device.reset_dev)


def main():
    define_os_specific_params()
    root = tk.Tk()
    root.title("Config window")
    root.geometry("800x600")
    root.configure(padx=10, pady=10)

    console = Text(root, wrap="word")
    console.grid(row=4, column=0, columnspan=2, padx=5, pady=5, sticky="nsew")
    button1 = Button(root,
                     text="Configure device",
                     bg="lightblue",
                     fg="black",
                     width=15,
                     height=2,
                     font=("Arial", 12),
                     command=lambda: config_process(console_output=console))
    button4 = Button(root,
                     text="Reset device",
                     bg="lightcoral",
                     fg="black",
                     width=15,
                     height=2,
                     font=("Arial", 12),
                     command=lambda: serial_parallel_process(target=Device.reset_dev))
    button3 = Button(root,
                     text="Start device",
                     bg="lightgreen",
                     fg="black",
                     width=15,
                     height=2,
                     font=("Arial", 12),
                     command=lambda: serial_parallel_process(target=Device.start_dev))
    button2 = Button(root,
                     text="Import config",
                     bg="lightblue",
                     fg="black",
                     width=15,
                     height=2,
                     font=("Arial", 12),
                     command=lambda: import_config(console))

    root.grid_rowconfigure(0, weight=1)
    root.grid_rowconfigure(1, weight=1)
    root.grid_rowconfigure(2, weight=1)
    root.grid_rowconfigure(3, weight=1)
    root.grid_rowconfigure(4, weight=4)

    root.grid_columnconfigure(0, weight=2)
    root.grid_columnconfigure(1, weight=2)

    button1.grid(row=0, column=0, padx=5, pady=5, sticky="nsew")
    button2.grid(row=0, column=1, padx=5, pady=5, sticky="nsew")
    button3.grid(row=1, column=0, padx=5, pady=5, sticky="nsew")
    button4.grid(row=1, column=1, padx=5, pady=5, sticky="nsew")

    root.mainloop()

    sys.stdout = sys.__stdout__



import re
import os
from tkinter import Text
import tkinter as tk
from .Device import Device
from .smartbadgecfgdict import config_dict


class Config:
    def get_config_value_from_cfg(parameter: int, line: str) -> int:
        if parameter is not None:
            pattern = r"config set %d (.*)" % parameter
            p = re.compile(pattern)
            match = p.search(line)
            if match:
                return int(match.group(1))

    def get_config_parameter_from_cfg(line: str) -> int:
        p = re.compile("config set (.*) ")
        match = p.search(line)
        if match:
            return int(match.group(1))

    def check_config_discrepancy(serial_port: str, br: int, console_output: Text) -> bool:
        device_config = Device.config_show_at_device(serial_port=serial_port, br=br)
        deveui = str(Device.get_deveui(serial_port=serial_port, br=br))
        config_file = os.path.join(os.path.join(os.path.dirname(__file__), "config"), "config.cfg")
        with open(config_file, 'r') as config:
            for line in config:
                config_parameter_cfg = Config.get_config_parameter_from_cfg(line)
                config_value_cfg = Config.get_config_value_from_cfg(config_parameter_cfg, line)
                config_name = config_dict.get(config_parameter_cfg)
                if config_parameter_cfg is not None or config_value_cfg is not None:
                    config_value_dev = Device.get_config_value_from_dev(device_config, config_name)

                    if config_parameter_cfg is 249 and config_value_dev is 5:
                        console_output.insert(tk.END, f"Config error: {deveui} \n")
                        console_output.insert(tk.END, f"An error occurred. Please try starting the device, then configuring again. \n")
                        return False

                    if config_value_cfg != config_value_dev:
                        console_output.insert(tk.END, f"Config error: {deveui} \n")
                        console_output.insert(tk.END, f"[Parameter : {config_name}] - Current: [{config_value_dev}] | Correct: [{config_value_cfg}] \n")
                        return False

        console_output.insert(tk.END, f"Done: {deveui} \n")
        return True



import re
import os
from serial import Serial
from time import sleep


class Device:
    def reset_dev(serial_port: str, br: int) -> None:
        with Serial(serial_port, br, timeout=1) as ser:
            ser.write(b'123\r')
            ser.write(b'system reset\r')
            ser.close()

    def start_dev(serial_port: str, br: int) -> None:
        with Serial(serial_port, br, timeout=1) as ser:
            ser.write(b'123\r')
            ser.write(b'system skip\r')
            sleep(6)
            ser.write(b'system log off\r')
            ser.close()

    def get_deveui(serial_port: str, br: int) -> int:
        with Serial(serial_port, br, timeout=1) as ser:
            ser.write(b'123\r')
            ser.write(b'123\r')
            ser.write(b'lora info\r')
            output = ser.read(1000).decode('utf-8')
            p = re.compile(r"DevEUI: (.*)")
            deveui = p.search(output)
            if deveui is not None:
                return deveui.group(1)

    def set_config_on_device(serial_port: str, br: int) -> None:
        with Serial(serial_port, br, timeout=1) as ser:
            ser.write(b'123\r')
            config_file = os.path.join(os.path.join(os.path.dirname(os.path.abspath(__file__)), "config"), "config.cfg")
            with open(config_file, 'rb') as config:
                for line in config:
                    ser.write(line.strip())
                    ser.write(b'\r')
            ser.write(b'config save\r')
            ser.write(b'system buzzer 6\r')
            ser.close()

    # This doesn't actually talk to the device directly, rather it just grabs the value from a string
    # Might move it back to the main module
    def get_config_value_from_dev(config_name: str, parameter: int) -> int:
        if parameter is not None:
            match_line = re.search(r".*\s+%s\s*=\s*(-?\d+)" % parameter, config_name)
            if match_line is not None:
                return int(match_line.group(1))

    def config_show_at_device(serial_port: str, br: int) -> str:
        with Serial(serial_port, br, timeout=1) as ser:
            ser.write(b'123\r')
            ser.write(b'config show\r')
            output = ser.read(16000)
            ser.close()
            return output.decode('utf-8')



config_dict = {
    0: 'ul_period',
    1: 'lora_period',
    2: 'pw_stat_period',
    3: 'periodic_pos_period',
    4: 'unknown',
    5: 'geoloc_sensor',
    6: 'geoloc_method',
    7: 'antenna',
    8: 'motion_nb_pos',
    9: 'gps_timeout',
    10: 'agps_timeout',
    11: 'gps_ehpe',
    12: 'gps_convergence',
    13: 'config_flags',
    14: 'transmit_strat',
    15: 'ble_beacon_cnt',
    16: 'ble_beacon_timeout',
    17: 'gps_standby_timeout',
    18: 'confirmed_ul_bitmap',
    19: 'confirmed_ul_retry',
    20: 'motion_sensitivity',
    21: 'shock_detection',
    22: 'periodic_activity_period',
    23: 'motion_duration',
    24: 'geofencing_scan_period',
    25: 'geofencing_collect_period',
    26: 'ble_rssi_filter',
    27: 'temperature_high',
    28: 'temperature_low',
    29: 'temperature_action',
    30: 'transmit_strat_custom',
    31: 'network_timeout_check',
    32: 'network_timeout_reset',
    33: 'collection_scan_type',
    34: 'collection_nb_entry',
    35: 'collection_ble_filter_type',
    36: 'collection_ble_filter_main_1',
    37: 'collection_ble_filter_main_2',
    38: 'collection_ble_filter_sec_value',
    39: 'collection_ble_filter_sec_mask',
    40: 'battery_capacity',
    41: 'reed_switch_configuration',
    42: 'gnss_constellation',
    43: 'prox_scan_pwr_min',
    44: 'prox_distance_coef',
    45: 'prox_scan_frequency',
    46: 'prox_backtrace_max_age',
    47: 'prox_distance_sliding_window',
    48: 'prox_exposure_50',
    49: 'prox_exposure_100',
    50: 'prox_exposure_150',
    51: 'prox_exposure_200',
    52: 'prox_exposure_250',
    53: 'prox_exposure_300',
    54: 'prox_exposure_400',
    55: 'prox_alarm_dist_immediate',
    56: 'prox_alarm_exposure',
    57: 'prox_warn_dist_immediate',
    58: 'prox_warn_exposure',
    59: 'prox_record_dist_immediate',
    60: 'prox_record_exposure',
    61: 'prox_alarm_buz_duration',
    62: 'prox_warn_buz_duration',
    63: 'prox_contact_policy',
    64: 'prox_scan_duration',
    65: 'prox_scan_window',
    66: 'prox_scan_interval',
    67: 'prox_alarm_remanence',
    68: 'prox_warn_remanence',
    69: 'prox_bcn_repeat',
    70: 'prox_bcn_tx_power',
    71: 'prox_reminder_period',
    72: 'prox_reminder_distance',
    73: 'prox_warn_disable_dist',
    74: 'prox_alarm_disable_dist',
    75: 'prox_max_speed_filter',
    76: 'prox_max_update',
    77: 'position_ble_filter_type',
    78: 'position_ble_filter_main_1',
    79: 'position_ble_filter_main_2',
    80: 'position_ble_filter_sec_value',
    81: 'position_ble_filter_sec_mask',
    82: 'position_ble_report_type',
    83: 'buzzer_volume',
    84: 'angle_detect_mode',
    85: 'angle_ref_acq',
    86: 'angle_ref_acc_x',
    87: 'angle_ref_acc_y',
    88: 'angle_ref_acc_z',
    89: 'angle_critical',
    90: 'angle_critical_hyst',
    91: 'angle_report_mode',
    92: 'angle_report_period',
    93: 'angle_report_repeat',
    94: 'angle_rising_time',
    95: 'angle_falling_time',
    96: 'angle_learning_time',
    97: 'angle_acc_accuracy',
    98: 'angle_deviation_delta',
    99: 'angle_deviation_min_interval',
    100: 'angle_deviation_max_interval',
    101: 'default_profile',
    102: 'password',
    103: 'gps_t0_timeout',
    104: 'gps_fix_timeout',
    105: 'geofencing_scan_duration',
    106: 'beaconing_type',
    107: 'beaconing_tx_power',
    108: 'beaconing_static_interval',
    109: 'beaconing_motion_interval',
    110: 'beaconing_motion_duration',
    111: 'ble_cnx_adv_duration',
    112: 'beacon_id_0',
    113: 'beacon_id_1',
    114: 'beacon_id_2',
    115: 'beacon_id_3',
    116: 'beacon_id_4',
    117: 'sos_period',
    118: 'motion_debounce',
    119: 'button_mapping',
    120: 'default_datarate',
    121: 'gps_ehpe_motion',
    122: 'gps_convergence_motion',
    123: 'gps_t0_timeout_motion',
    245: 'ble_cli_active',
    246: 'profile',
    247: 'consumption',
    248: 'ble_bond_info',
    249: 'mode',
    250: 'acc_x_axis',
    251: 'acc_y_axis',
    252: 'acc_z_axis',
    253: 'ble_version',
    254: 'firmware_version',
}



from setuptools import setup, find_packages

with open("README.md", "r") as f:
    description = f.read()

setup(
    name="AbeewayConfig",
    version="0.2.2.2",
    description="Abeeway configuration tool",
    author="João Lucas",
    url="https://github.com/jlabbude/AbeewayConfig",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    install_requires=[
        "pyserial",
        "tk",
    ],
    entry_points={
        "console_scripts": [
            "abeewayconfig = AbeewayConfig.abeewayconfig:main",
        ],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
    ],
    python_requires='>=3.0',
    long_description=description,
    long_description_content_type="text/markdown",
)
